@@ -20,9 +20,9 @@ import numpy.random as nr
 import theano
 
 from theano.gradient import verify_grad as tt_verify_grad
-from theano.sandbox.rng_mrg import MRG_RandomStreams
+from theano.sandbox.rng_mrg import MRG_RandomStream as RandomStream
 
-from ..theanof import set_tt_rng, tt_rng
+from pymc3.theanof import set_tt_rng, tt_rng
 
 
 class SeededTest:
@@ -35,7 +35,7 @@ class SeededTest:
     def setup_method(self):
         nr.seed(self.random_seed)
         self.old_tt_rng = tt_rng()
-        set_tt_rng(MRG_RandomStreams(self.random_seed))
+        set_tt_rng(RandomStream(self.random_seed))
 
     def teardown_method(self):
         set_tt_rng(self.old_tt_rng)
