@@ -38,10 +38,9 @@ from pymc3 import (
     sample,
 )
 from pymc3.distributions.shape_utils import to_tuple
+from pymc3.tests.helpers import SeededTest
 from pymc3.theanof import floatX
 
-from .helpers import SeededTest
-
 
 # Generate data
 def generate_normal_mixture_data(w, mu, sd, size=1000):
